/*
	Software Copyright (c) 1985-2011 Information Transfer LLP

	Seminar 7.5.3.914 - PLATINUM

*/

//Returns the scale value for use in the viewport <meta> tag for mobile device re-scaling.
function getMobileScale(moduleWidth, moduleHeight)
{
	//Default scale is 1.0
	var scale = 1.0;

	//Try to workout the appropriate scale for mobile devices
	//(window.orientation should only be defined for mobile devices, not desktops)
	if( window && (typeof window.orientation == 'number') )
	{
		//Device detection
		var isIPad = false;
		var isIPhone = false;
		var isAndroid = false;
		if(navigator && navigator.userAgent && navigator.userAgent.match(/iPad/i)) isIPad = true;
		if(navigator && navigator.userAgent && navigator.userAgent.match(/iPhone|iPod/i)) isIPhone = true;
		if(navigator && navigator.userAgent && navigator.userAgent.match(/android/i)) isAndroid = true;
	
		//Get the orientation of the module
		var moduleOrientation = ( moduleHeight > moduleWidth ? "portrait" : "landscape");	

		//Get the orientation of the device
		var deviceOrientation = "landscape";
		if ( isIPad || isIPhone ) {
			deviceOrientation = ( Math.abs(window.orientation) == 0 ? "portrait" : "landscape" );
		}
		else {
			//Any other devices, inc. androids
			deviceOrientation = ( Math.abs(window.orientation) == 0 ? "landscape" : "portrait" );
		}
			
		//If the device orientation is not the same as the module's orientation, then
		//pretend the device is in the same orientation as the module. This means the
		//user will have to turn them phone to match the design of the module.
		if ( deviceOrientation != moduleOrientation ) 
		{
			//alert('Please rotate your device'); //Can be commented out
			deviceOrientation = moduleOrientation; 
		}

		//Get the available width and height
		var availableWidth = moduleWidth;
		var availableHeight = moduleHeight;		

		//Get the actual screen size (not the size available inside the browser, 
		//because the available browser size is faked to pretend it is
		//bigger than it actually is
		if ( window && window.screen && window.screen.availWidth ) availableWidth =  window.screen.availWidth;
		if ( window && window.screen && window.screen.availHeight ) availableHeight =  window.screen.availHeight;

		//Because iPhones and iPads always give available portrait width and height, make sure that we
		//swap them if the proportions don't agree with the portait/landscape mode
		if ( ( deviceOrientation == "landscape" && availableWidth < availableHeight ) || ( deviceOrientation == "portrait" && availableHeight < availableWidth ) ) {
			var tmpWidth = availableWidth;
			availableWidth = availableHeight;
			availableHeight = tmpWidth;
		} 

		//Account for the height taken by the browser chrome
		if ( isIPhone ) {
			availableHeight -= 20; //Top status bar
			availableHeight -= (deviceOrientation == 'landscape' ? 32 : 44); //Bottom tools bar
		}
		else if (isIPad ) {
			//For iPad
			availableHeight -= 96; //Top bar, inc. status, address and tab bars
		}
		else {
			//For all others, inc. Androids. This is a guess value in between iPhone and iPad.
			availableHeight -= 75;
		}
		
		//Apply the rescaling formula
		if ( availableWidth < moduleWidth || availableHeight < moduleHeight )
		{
			scale = Math.min(availableWidth/moduleWidth,availableHeight/moduleHeight);
		}
	}

	return scale;
}
