/*

Seminar Web Interface v1.3 (BETA)

Copyright (c) Information Transfer LLP 2012

*/

var SWI = (function() {
// private variables	
	var my = {},
	initialized = false,
	cw = null,
	pw = null,
	errorCode = 0,
	pageHandles = [],
	sectionHandles = [],
	pageChangeNotifications = [],
	userVariables = [],
	scormVariablesFlag = false,
	buttonNameIdTable = {
	"CONT" : "Continue",
	"FIN" : "Finish",
	"FWD" : "Forward",
	"BACK" : "Back",
	"EJECT" : "Eject",
	"REF" : "Ref",
	"SUBMIT" : "submit"
	};
;
	
// private functions

	function assignHandleForPageObj(page) {
		var i = pageHandles.length;
		pageHandles.push(page);
		if(pageHandles.length === 1)      // special case to avoid Handle = 0
		{
			pageHandles.push(page);
			return 1;
		}
		return i;
	};
	
	function getPageObjForHandle(handle) {
		if(handle)
		{
			if(handle < pageHandles.length) return pageHandles[handle];
			errorCode = 2;
		}
		else return cw.currentPage;
	};
	
	function assignHandleForSectionObj(section) {
		var i = sectionHandles.length;
		sectionHandles.push(section);
		if(sectionHandles.length === 1)      // special case to avoid Handle = 0
		{
			sectionHandles.push(section);
			return 1;
		}
		return i;
	};
	
	function getSectionObjForHandle(handle) {
		if(handle)
		{
			if(handle < sectionHandles.length) return sectionHandles[handle];
			errorCode = 2;
		}
		else 
		{
			if(cw.currentPage) return cw.currentPage.sectionObject;
			errorCode = 7
		}
	};

	function getIndexForSectionHandle(handle) {
		errorCode = 0;
		var s = getSectionObjForHandle(handle);
		if(errorCode) return -1;
		for(var i = 0; i < cw.sections.length; i++)
		{
			if(cw.sections[i] == s) return i;
		}
		return -1;
	};
	
	function recalcSectionQuestionCount(section) {
		section.questionCount = 0;
		for(var i = 0; i < section.pages.length; i++)
		{
			var p = section.pages[i];
			if(p.isQuestion && p.isQuestionCount) section.questionCount++;
		}
	};

	function getButtonNameMapping(buttonName)
	{
		return buttonNameIdTable[buttonName];
	};
	
// public methods ... note declared as method of the "my" object

	my.getVersion = function( ) {
		return "1.3";
	};
	
	my.getLastError = function() {
		var e = errorCode;
		errorCode = 0;
		return e;
	};
	
// Module functions

	my.getExamReviewMode = function() {
		return cw.examModeReview;
	}
	
	my.setExamReviewMode = function(value) {
		cw.examModeReview = value;
	}
	
	
// Page related functions	
	
	my.getPageByFilename = function(filename) {
		for(var i=1; i < pageHandles.length; i++) // skipping over i=0 because that is "current page"
		{
			if(pageHandles[i].filename === filename) return i;
		}
		var p = cw.controlGetPageByFilename(filename);
		if(p) return assignHandleForPageObj(p);
		return -1;
		errorCode = 1;
	};

	my.getPageByPageID = function(pageID) {
		for(var i=1; i < pageHandles.length; i++) // skipping over i=0 because that is "current page"
		{
			if(pageHandles[i].pageID === pageID) return i;
		}
		var p = cw.controlGetPageByID(pageID);
		if(p) return assignHandleForPageObj(p);
		return -1;
		errorCode = 1;
	};
	
	my.getCurrentPage = function() {
		if(cw.currentPage)
		{
			return my.getPageByPageID(cw.currentPage.pageID);
		}
	};

	my.getPageByTagID = function(tagID) {
		for(var i=1; i < pageHandles.length; i++) // skipping over i=0 because that is "current page"
		{
			if(pageHandles[i].tagID === tagID) return i;
		}
		var p = cw.controlGetPageByTagID(tagID);
		if(p) return assignHandleForPageObj(p);
		return -1;
		errorCode = 1;
	};
	
	my.getPageID = function(handle) {
		var p = getPageObjForHandle(handle);
		if(p) return p.pageID;
	};

	my.getPageTitle = function(handle) {
		var p = getPageObjForHandle(handle);
		if(p) return p.title;
	};
		
	my.getPageIsQuestion = function(handle) {
		var p = getPageObjForHandle(handle);
		if(p) return p.isQuestion;
	};
	
	my.getPageQuestionType = function(handle) {
		var p = getPageObjForHandle(handle);
		if(p) return p.questionType;
	};
	
	my.setPageQuestionType = function(handle, questionType) {
		var p = getPageObjForHandle(handle);
		if(p)
		{
			var was = p.questionType;
			p.questionType = questionType;
			if(questionType) 
			{
				p.isQuestionCount = true;
				p.isQuestion = true;
				p.student = "";
				p.correct = "";
			}
			else 
			{
				p.isQuestionCount = false;
				p.isQuestion = false;
				delete p.student;
				delete p.correct;
			}
			recalcSectionQuestionCount(p.sectionObject);
			if(was != questionType && p.runMode > 0)
			{
				if(was) p.sectionObject.questionsReq--;
				if(questionType) p.sectionObject.questionsReq++;
			}
			return true;
		}
	};
		
	my.getPageIsQuestionCount = function(handle) {
		var p = getPageObjForHandle(handle);
		if(p)
		{
			if(p.isQuestion) return p.isQuestionCount;
			else return false;
		}
	};
	
	my.setPageQuestionAttempt = function(handle,attemptStr,percent) {
		var p = getPageObjForHandle(handle);
		if(p)
		{
			if(p.isQuestion) 
			{
				pw.scoresPostAttemptPercent(attemptStr,percent,p);
				return true;
			}
			errorCode = 3;
		};
	};
	
	my.setPageQuestionCorrect = function(handle,correctStr) {
		var p = getPageObjForHandle(handle);
		if(p)
		{
			if(p.isQuestion) 
			{
				p.correct = correctStr;
				return true;
			}
			errorCode = 3;
		}
	};

	my.getPageQuestionAttempted = function(handle) {
		var p = getPageObjForHandle(handle);
		if(p)
		{
			if(p.isQuestion) return p.attempted;
			errorCode = 3;
		}
	};
	
	my.getPageStudentAnswer = function(handle) {
		var p = getPageObjForHandle(handle);
		if(p)
		{
			if(p.isQuestion) return p.student;
			errorCode = 3;
		}
	};
	
	my.getPageQuestionScore = function(handle) {
		var p = getPageObjForHandle(handle);
		if(p)
		{
			if(p.isQuestion) return cw.controlGetPagePoints(p); // see FB #3179
			errorCode = 3;
		}
	};

//	Button functions etc.

	my.hideContinueButton = function( ) {
		if(cw.currentPage)
		{
			pw.coreHideObjectId("continuebtndiv");
			return true;
		}
		errorCode = 4;
	};

	my.showContinueButton = function( ) {
		if(cw.currentPage)
		{
			pw.coreShowObjectId("continuebtndiv");
			return true;
		}
		errorCode = 4;
	};

	my.isContinueButtonVisible = function( ) {
		if(cw.currentPage)
		{
			var cb = pw.document.getElementById("continuebtndiv");
			if(cb)
			{
				if(cb.style.style.visibility === "visible") return true;
				else return false;
			}
			errorCode = 5;
		}
		errorCode = 4;
	};
	
	my.disableContinueButton = function() {
		if(cw.currentPage)
		{
			pw.coreDisableContinue();
			return true;
		}
		errorCode = 4;
	};
	
	my.enableContinueButton = function() {
		if(cw.currentPage)
		{
			pw.coreEnableContinue();
			return true;
		}
		errorCode = 4;
	};
	
	my.getContinueButtonIsDisabled = function() {
		if(cw.currentPage)
		{
			return pw.coreGetBtnIsDisabled(pw.document.getElementById("Continue"));
		}
		errorCode = 4;
	};

	my.setButtonVisible = function(buttonName, isVisible) {
		if(cw.currentPage)
		{
			var buttonId = getButtonNameMapping(buttonName);
			if(isVisible)
			{
				pw.coreShowObjectId(buttonId);
			}
			else
			{
				pw.coreHideObjectId(buttonId);
			}
			return true;
		}
		errorCode = 4;
	};

	my.setButtonEnable = function(buttonName, isEnable) {
		if(cw.currentPage)
		{
			var buttonId = getButtonNameMapping(buttonName);
			if(isEnable)
			{
				pw.coreEnableButtonId(buttonId);
			}
			else
			{
				pw.coreDisableButtonId(buttonId);
			}
			return true;
		}
		errorCode = 4;
	};
	
	my.getButtonOnClick = function(buttonName) {
		if(cw.currentPage)
		{
			var buttonId = getButtonNameMapping(buttonName);
			var	button = pw.document.getElementById(buttonId);
			if(button)
			{
				return button.onclick;
			}
			else
			{
				errorCode = 11;
				return null;
			}
		}
		errorCode = 4;
	};
	
	my.setButtonOnClick = function(buttonName, onClickHandler) {
		if(cw.currentPage)
		{
			var buttonId = getButtonNameMapping(buttonName);
			var	button = pw.document.getElementById(buttonId);
			if(button)
			{
				button.onclick = onClickHandler;
			}
			else
			{
				errorCode = 11;
				return false;
			}
			return true;
		}
		errorCode = 4;
	};

	function addToButtonArray(buttons, buttonId) {
		var already = false;
		for(var i = 0; i < buttons.length; i++)
		{
			if(p.buttons[i] == buttonId) already = true;
		}
		if(!already) buttons.push(buttonId);
	};

	my.setButtonHiddenInitially = function(handle, buttonName) {
		var p = getPageObjForHandle(handle);
		if(p)
		{
			if(!p.initialHiddenBtns) p.initialHiddenBtns = new Array();
			var buttonId = getButtonNameMapping(buttonName);
			addToButtonArray(p.initialHiddenBtns, buttonId);
		}
	};

	my.setButtonDisabledInitially = function(handle, buttonName) {
		var p = getPageObjForHandle(handle);
		if(p)
		{
			if(!p.initialDisabledBtns) p.initialDisabledBtns = new Array();
			var buttonId = getButtonNameMapping(buttonName);
			addToButtonArray(p.initialDisabledBtns, buttonId);
		}
	};

// Question button stuff - could/should be better than this
	my.getQuestionButtonOnClick = function(buttonName) {
		var b = pw.document.getElementById(buttonName); // this may need a lookup table in future versions
		if(b) return b.onclick;
		else 
		{
			errorCode = 10;
			return null;
		}
	};

	my.setQuestionButtonOnClick = function(buttonName,onClickHandler) {
		var b = pw.document.getElementById(buttonName); // this may need a lookup table in future versions
		if(b)
		{
			b.onclick = onClickHandler;
			return true;
		}
		else 
		{
			errorCode = 10;
			return false;
		}
	};

	my.hideQuestionButton = function(buttonName) {
		var b = pw.document.getElementById(buttonName); // this may need a lookup table in future versions
		if(b)
		{
			pw.coreHideObjectId(buttonName);
			return true;
		}
		else 
		{
			errorCode = 10;
			return false;
		}
	};

	my.showQuestionButton = function(buttonName) {
		var b = pw.document.getElementById(buttonName); // this may need a lookup table in future versions
		if(b)
		{
			pw.coreShowObjectId(buttonName);
			return true;
		}
		else 
		{
			errorCode = 10;
			return false;
		}
	};

//  private function to generalise the mapping from branchValue A thru Z to the DOM ElementID	
	function getBranchElementByID(branchValue)
	{
		try
		{
			return pw.document.getElementById("branch" + branchValue);
		}
		catch(e)
		{
			return null;
		}
	}
	
// Branching page button stuff - specific because Branching pages are navigation	
	my.getBranchButtonOnClick = function(branchValue) {
		var b = getBranchElementByID(branchValue);
		if(b) return b.onclick;
		else 
		{
			errorCode = 10;
			return null;
		}
	};

	my.setBranchButtonOnClick = function(branchValue,onClickHandler) {
		var b = getBranchElementByID(branchValue);
		if(b)
		{
			b.onclick = onClickHandler;
			return true;
		}
		else 
		{
			errorCode = 10;
			return false;
		}
	};

	my.hideQuestionButton = function(buttonName) {
		var b = getBranchElementByID(branchValue);
		if(b)
		{
			pw.coreHideObjectId(buttonName);
			return true;
		}
		else 
		{
			errorCode = 10;
			return false;
		}
	};

	my.showQuestionButton = function(buttonName) {
		var b = getBranchElementByID(branchValue);
		if(b)
		{
			pw.coreShowObjectId(buttonName);
			return true;
		}
		else 
		{
			errorCode = 10;
			return false;
		}
	};
	
	
// Section navigation etc.

	my.getSectionByIndex = function(index) {
		if(index >=0 && index < cw.sections.length)
		{
			return assignHandleForSectionObj(cw.sections[index]);
		}
		errorCode = 8;
	};
	
	my.getSectionForPage = function(handle) {
		var p = getPageObjForHandle(handle);
		if(p)
		{
			if(p.sectionObject) return assignHandleForSectionObj(p.sectionObject);
			errorCode = 9;
		}
	};
	
	my.getSectionTitle = function(handle) {
		var s = getSectionObjForHandle(handle);
		if(s)
		{
			return s.Title;
		}
		else
		{
			errorCode = 6;
			return null;
		}
	};

	my.getSectionRunMode = function(handle) {
		var s = getSectionObjForHandle(handle);
		if(s)
		{
			return s.runMode;
		}
		else
		{
			errorCode = 6;
			return null;
		}
	};

	my.getSectionPageCount = function(handle) {
		var s = getSectionObjForHandle(handle);
		if(s)
		{
			return s.pages.length;
		}
		else
		{
			errorCode = 6;
			return null;
		}	
	};

	my.getSectionQuestionCount = function(handle) {
		var s = getSectionObjForHandle(handle);
		if(s)
		{
			return s.questionCount;
		}
		else
		{
			errorCode = 6;
			return null;
		}
	};

	my.getSectionQuestionsReq = function(handle) {
		var s = getSectionObjForHandle(handle);
		if(s)
		{
			return s.questionsReq;
		}
		else
		{
			errorCode = 6;
			return null;
		}	
	};
	
	my.getSectionOptional = function(handle) {
		var s = getSectionObjForHandle(handle);
		if(s)
		{
			return s.optional;
		}
		else
		{
			errorCode = 6;
			return null;
		}
	};
	
	my.setSectionOptional = function(handle,value) {
		var s = getSectionObjForHandle(handle);
		if(s)
		{
			s.optional = value;
		}
		else
		{
			errorCode = 6;
		}
	};
	
	my.getSectionSealed = function(handle) {
		var s = getSectionObjForHandle(handle);
		if(s)
		{
			return s.controlled;
		}
		else
		{
			errorCode = 6;
			return null;
		}
	};
	
	my.setSectionSealed = function(handle,value) {
		var s = getSectionObjForHandle(handle);
		if(s)
		{
			s.controlled = value;
		}
		else
		{
			errorCode = 6;
		}
	};
	
	my.getSectionControlledBy = function(handle) {
		var s = getSectionObjForHandle(handle);
		if(s)
		{
			return s.controlSection;
		}
		else
		{
			errorCode = 6;
			return null;
		}
	};
	
	my.setSectionControlledBy = function(handle,value) {
		var s = getSectionObjForHandle(handle);
		if(s)
		{
			s.controlSection = value;
		}
		else
		{
			errorCode = 6;
		}
	};

//	Page level navigation

	my.getPageNextPage = function(handle) {
		var p = getPageObjForHandle(handle);
		if(p)
		{
			var n = p.nextPage;
			return assignHandleForPageObj(n);
		}
	};

	my.setPageNextPage = function(handle,handleNext) {
		var p = getPageObjForHandle(handle);
		if(p)
		{
			var n = getPageObjForHandle(handleNext);
			p.nextPage = n;
		}
	};

	my.getPagePrevPage = function(handle) {
		var p = getPageObjForHandle(handle);
		if(p)
		{
			var n = p.prevPage;
			return assignHandleForPageObj(n);
		}
	};

	my.setPagePrevPage = function(handle,handlePrev) {
		var p = getPageObjForHandle(handle);
		if(p)
		{
			var n = getPageObjForHandle(handlePrev);
			p.nextPage = n;
		}
	};
	
	my.loadPage = function(handle) {
		var p = getPageObjForHandle(handle);
		if(p)
		{
			cw.controlLoadPage(p);
		}
	};

	my.nextPage = function(handle) {
		if(!handle)
		{
			cw.controlNextPage();
		}
		else
		{
			var p = getPageObjForHandle(handle);
			if(p)
			{
				cw.controlNextPage(p);
			}
		}
	};

	my.prevPage = function(handle) {
		if(!handle)
		{
			cw.controlPrevPage();
		}
		else
		{
			var p = getPageObjForHandle(handle);
			if(p)
			{
				cw.controlPrevPage(p);
			}
		}
	};

	my.forwardPage = function(handle) {
		cw.controlForwards();
	};

	my.backwardPage = function(handle) {
		cw.controlBackwards();
	};

	my.loadSectionFirstPage = function(handle) {
		var idx = getIndexForSectionHandle(handle);
		if(!errorCode) cw.controlGotoSection(idx);
	};

	my.registerPageChangeNotification = function(notify) {
		var notThere = true;
		for(var i = 0; i < pageChangeNotifications.length; i++)
		{
			if(pageChangeNotifications[i] === notify)
			{
				notThere = false;
			}
		}
		if(notThere)
		{
			pageChangeNotifications.push(notify);
		}
	};

	my.unregisterPageChangeNotification = function(notify) {
		for(var i = 0; i < pageChangeNotifications.length; i++)
		{
			if(pageChangeNotifications[i] === notify)
			{
				pageChangeNotifications.pop(i);
				break;
			}
		}
	};

	my.pageChanged = function(phase) {
		for(var i = 0; i < pageChangeNotifications.length; i++)
		{
			pageChangeNotifications[i](phase);
		}
	};

	my.changeMenuPageTo = function(handle) {
		var p = getPageObjForHandle(handle);
		if(p)
		{
			if(!cw.menuPage)
			{
				cw.menuPage = new Object();
				cw.menuPage.ismenu = true;
				cw.menuPage.section = -1;
				cw.menuPage.isQuestion = false;		
			}
			cw.menuPage.filename = p.filename;
			cw.menuPage.pageID = p.pageID;
			cw.menuPage.title = p.title;
		}
	};

// Completion stuff

	my.setSectionComplete = function(handle) {
		var s = getSectionObjForHandle(handle);
		if(s)
		{
			s.completed = true;
		}
		else { errorCode = 6; }
	};

	my.getSectionComplete = function(handle) {
		var s = getSectionObjForHandle(handle);
		if(s)
		{
			return s.completed;
		}
		else { errorCode = 6; }
	};

	my.setSectionNotComplete = function(handle) {
		var s = getSectionObjForHandle(handle);
		if(s)
		{
			s.completed = false;
		}
		else { errorCode = 6; }
	};

	my.hitSectionFinishingPosts = function(handle) {
		var s = getSectionObjForHandle(handle);
		if(s)
		{
			var l = s.pages.length;
			for(var i = 0; i < l; i++)
			{
				var p = s.pages[i];
				p.loaded = true;
				cw.controlDoFinishingPost(p);
			}
		}
		else { errorCode = 6; }
	};

	my.getModuleComplete = function() {
		return cw.controlGetModuleComplete();
	};

	my.setModuleComplete = function() {
		cw.controlFinishAllSections();
	};

	my.getModuleScore = function() {
		return cw.controlGetModuleScore();
	};

	my.getModuleStatus = function() {
		return cw.controlGetModuleStatus();
	};

	my.controlGetModuleDuration = function() {
		return cw.controlGetModuleDurationSeconds();
	};

//	SCORM stuff for pageAPI

	my.pageAPISetCMIData = function(cmiData) {
		if(cw.currentPage)
		{
			if(!cw.currentPage.cmi) cw.currentPage.cmi = {};
			var name;
			for(name in cmiData)
			{
				if(typeof cmiData[name] !== "function")
				{
					cw.currentPage.cmi[name] = cmiData[name];
				}
			}
		}
		else errorCode = 4;
	};

	my.pageAPIGetCMIData = function() {
		var r = {};
		if(cw.currentPage)
		{
			if(cw.currentPage.cmi)
			{
				var name;
				for(name in cw.currentPage.cmi)
				{
					if(typeof cw.currentPage.cmi[name] !== "function")
					{
						r[name] = cw.currentPage.cmi[name];
					}
				}
			}
		}
		else errorCode = 4;
		return r;
	};

	my.pageAPISetValue = function(name,value) {
		if(cw.currentPage)
		{
			if(!cw.currentPage.cmi) cw.currentPage.cmi = {};
			cw.currentPage.cmi[name] = value;
		}
		else errorCode = 4;
	};

//  Variable things

	my.setVariable = function(name,value) {
		userVariables[name] = value;
	};
	
	my.getVariable = function(name) {
		return userVariables[name];
	};
	
	my.getAllVariables = function() {
		var data = "";
		for(var i in userVariables)
		{
			data = data + i + "=" + userVariables[i] + ";"; 
		}
		return data;
	};
	
	my.setAllVariables = function(data, isScorm) {
		if(isScorm) scormVariablesFlag = true;
		if(data) 
		{
			var unsave = data.split(";");
			var	bit = "";
			for(var i = 0; i < unsave.length; i++)
			{
				bit = unsave[i].split("=");
				userVariables[bit[0]] = bit[1];
			}
		}
	};
	

//	Admin stuff - setup and teardown

	my.init = function(controlWindow,presentWindow) {
		if(!initialized)
		{
			if(controlWindow) cw = controlWindow;
			else cw = parent.controlWindow;
			if(presentWindow) pw = presentWindow;
			else pw = parent.presentWindow;
			cw.controlOnPageChange = my.pageChanged;
			initialized = true;
		}
	};

	return my;	
}()); // end of the object
